@@ -29,21 +29,23 @@
 
 #include <config.h>
 
-#include "rcsid.h"
-RCSID (PKG_VER "$Id: pwck.c,v 1.27 2005/05/25 18:20:25 kloczek Exp $")
-#include <stdio.h>
+#ident "$Id: pwck.c,v 1.35 2006/05/07 17:44:39 kloczek Exp $"
+
 #include <fcntl.h>
 #include <grp.h>
-#include "prototypes.h"
-#include "defines.h"
-#include "chkname.h"
 #include <pwd.h>
+#include <stdio.h>
+#include "chkname.h"
 #include "commonio.h"
+#include "defines.h"
+#include "prototypes.h"
 #include "pwio.h"
+#include "shadowio.h"
+#include "getdef.h"
+#include "nscd.h"
 extern void __pw_del_entry (const struct commonio_entry *);
 extern struct commonio_entry *__pw_get_head (void);
 
-#include "shadowio.h"
 extern void __spw_del_entry (const struct commonio_entry *);
 extern struct commonio_entry *__spw_get_head (void);
 
@@ -59,7 +61,7 @@ extern struct commonio_entry *__spw_get_head (void);
 #define	E_CANTUPDATE	5
 
 /*
- * Local variables
+ * Global variables
  */
 
 static char *Prog;
@@ -76,7 +78,6 @@ static int yes_or_no (void);
 /*
  * usage - print syntax message and exit
  */
-
 static void usage (void)
 {
 	fprintf (stderr, _("Usage: %s [-q] [-r] [-s] [passwd [shadow]]\n"),
@@ -87,7 +88,6 @@ static void usage (void)
 /*
  * yes_or_no - get answer to question from the user
  */
-
 static int yes_or_no (void)
 {
 	char buf[80];
@@ -97,14 +97,13 @@ static int yes_or_no (void)
 	 */
 
 	if (read_only) {
-		puts (_("No"));
+		printf (_("No\n"));
 		return 0;
 	}
 
 	/*
 	 * Get a line and see what the first character is.
 	 */
-
 	if (fgets (buf, sizeof buf, stdin))
 		return buf[0] == 'y' || buf[0] == 'Y';
 
@@ -114,12 +113,11 @@ static int yes_or_no (void)
 /*
  * pwck - verify password file integrity
  */
-
 int main (int argc, char **argv)
 {
 	int arg;
 	int errors = 0;
-	int deleted = 0;
+	int changed = 0;
 	struct commonio_entry *pfe, *tpfe;
 	struct passwd *pwd;
 	int sort_mode = 0;
@@ -131,19 +129,17 @@ int main (int argc, char **argv)
 	/*
 	 * Get my name so that I can use it to report errors.
 	 */
-
 	Prog = Basename (argv[0]);
 
 	setlocale (LC_ALL, "");
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	textdomain (PACKAGE);
 
-	OPENLOG ("pwsk");
+	OPENLOG ("pwck");
 
 	/*
 	 * Parse the command line arguments
 	 */
-
 	while ((arg = getopt (argc, argv, "eqrs")) != EOF) {
 		switch (arg) {
 		case 'e':	/* added for Debian shadow-961025-2 compatibility */
@@ -169,7 +165,6 @@ int main (int argc, char **argv)
 	/*
 	 * Make certain we have the right number of arguments
 	 */
-
 	if (optind != argc && optind + 1 != argc && optind + 2 != argc)
 		usage ();
 
@@ -177,7 +172,6 @@ int main (int argc, char **argv)
 	 * If there are two left over filenames, use those as the password
 	 * and shadow password filenames.
 	 */
-
 	if (optind != argc) {
 		pwd_file = argv[optind];
 		pw_name (pwd_file);
@@ -192,7 +186,6 @@ int main (int argc, char **argv)
 	/*
 	 * Lock the files if we aren't in "read-only" mode
 	 */
-
 	if (!read_only) {
 		if (!pw_lock ()) {
 			fprintf (stderr, _("%s: cannot lock file %s\n"),
@@ -216,7 +209,6 @@ int main (int argc, char **argv)
 	 * Open the files. Use O_RDONLY if we are in read_only mode, O_RDWR
 	 * otherwise.
 	 */
-
 	if (!pw_open (read_only ? O_RDONLY : O_RDWR)) {
 		fprintf (stderr, _("%s: cannot open file %s\n"),
 			 Prog, pwd_file);
@@ -244,13 +236,11 @@ int main (int argc, char **argv)
 	/*
 	 * Loop through the entire password file.
 	 */
-
 	for (pfe = __pw_get_head (); pfe; pfe = pfe->next) {
 		/*
 		 * If this is a NIS line, skip it. You can't "know" what NIS
 		 * is going to do without directly asking NIS ...
 		 */
-
 		if (pfe->line[0] == '+' || pfe->line[0] == '-')
 			continue;
 
@@ -259,22 +249,18 @@ int main (int argc, char **argv)
 		 * have no (struct passwd) entry because they couldn't be
 		 * parsed properly.
 		 */
-
 		if (!pfe->eptr) {
-
 			/*
 			 * Tell the user this entire line is bogus and ask
 			 * them to delete it.
 			 */
-
 			printf (_("invalid password file entry\n"));
-			printf (_("delete line `%s'? "), pfe->line);
+			printf (_("delete line '%s'? "), pfe->line);
 			errors++;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-
 			if (!yes_or_no ())
 				continue;
 
@@ -284,11 +270,10 @@ int main (int argc, char **argv)
 			 * list. When done, it skips back to the top of the
 			 * loop to try out the next list element.
 			 */
-
 		      delete_pw:
 			SYSLOG ((LOG_INFO, "delete passwd line `%s'",
 				 pfe->line));
-			deleted++;
+			changed++;
 
 			__pw_del_entry (pfe);
 			continue;
@@ -297,27 +282,23 @@ int main (int argc, char **argv)
 		/*
 		 * Password structure is good, start using it.
 		 */
-
 		pwd = pfe->eptr;
 
 		/*
 		 * Make sure this entry has a unique name.
 		 */
-
 		for (tpfe = __pw_get_head (); tpfe; tpfe = tpfe->next) {
 			const struct passwd *ent = tpfe->eptr;
 
 			/*
 			 * Don't check this entry
 			 */
-
 			if (tpfe == pfe)
 				continue;
 
 			/*
 			 * Don't check invalid entries.
 			 */
-
 			if (!ent)
 				continue;
 
@@ -328,15 +309,13 @@ int main (int argc, char **argv)
 			 * Tell the user this entry is a duplicate of
 			 * another and ask them to delete it.
 			 */
-
-			puts (_("duplicate password entry\n"));
-			printf (_("delete line `%s'? "), pfe->line);
+			printf (_("duplicate password entry\n"));
+			printf (_("delete line '%s'? "), pfe->line);
 			errors++;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-
 			if (yes_or_no ())
 				goto delete_pw;
 		}
@@ -352,7 +331,6 @@ int main (int argc, char **argv)
 		/*
 		 * Make sure the primary group exists
 		 */
-
 		if (!quiet && !getgrgid (pwd->pw_gid)) {
 
 			/*
@@ -367,13 +345,10 @@ int main (int argc, char **argv)
 		/*
 		 * Make sure the home directory exists
 		 */
-
 		if (!quiet && access (pwd->pw_dir, F_OK)) {
-
 			/*
 			 * Home directory doesn't exist, give a warning
 			 */
-
 			printf (_
 				("user %s: directory %s does not exist\n"),
 				pwd->pw_name, pwd->pw_dir);
@@ -383,18 +358,70 @@ int main (int argc, char **argv)
 		/*
 		 * Make sure the login shell is executable
 		 */
-
 		if (!quiet && pwd->pw_shell[0]
 		    && access (pwd->pw_shell, F_OK)) {
 
 			/*
 			 * Login shell doesn't exist, give a warning
 			 */
-
 			printf (_("user %s: program %s does not exist\n"),
 				pwd->pw_name, pwd->pw_shell);
 			errors++;
 		}
+#ifdef SHADOWPWD
+		/*
+		 * Make sure this entry exists in the /etc/gshadow file.
+		 */
+
+		if (is_shadow) {
+			spw = (struct spwd *) spw_locate (pwd->pw_name);
+			if (spw == NULL) {
+				printf (_
+					("no matching password file entry in %s\n"),
+					spw_file);
+				printf (_("add user '%s' in %s? "),
+					pwd->pw_name, spw_file);
+				errors++;
+				if (yes_or_no ()) {
+					struct spwd sp;
+					struct passwd pw;
+
+					sp.sp_namp = pwd->pw_name;
+					sp.sp_pwdp = pwd->pw_passwd;
+					sp.sp_min =
+					    getdef_num ("PASS_MIN_DAYS", -1);
+					sp.sp_max =
+					    getdef_num ("PASS_MAX_DAYS", -1);
+					sp.sp_warn =
+					    getdef_num ("PASS_WARN_AGE", -1);
+					sp.sp_inact = -1;
+					sp.sp_expire = -1;
+					sp.sp_flag = -1;
+					sp.sp_lstchg =
+					    time ((time_t *) 0) / (24L * 3600L);
+					changed++;
+
+					if (!spw_update (&sp)) {
+						fprintf (stderr,
+							 _
+							 ("%s: can't update shadow entry for %s\n"),
+							 Prog, sp.sp_namp);
+						exit (E_CANTUPDATE);
+					}
+					/* remove password from /etc/passwd */
+					pw = *pwd;
+					pw.pw_passwd = SHADOW_PASSWD_STRING;	/* XXX warning: const */
+					if (!pw_update (&pw)) {
+						fprintf (stderr,
+							 _
+							 ("%s: can't update passwd entry for %s\n"),
+							 Prog, pw.pw_name);
+						exit (E_CANTUPDATE);
+					}
+				}
+			}
+		}
+#endif
 	}
 
 	if (!is_shadow)
@@ -403,13 +430,18 @@ int main (int argc, char **argv)
 	/*
 	 * Loop through the entire shadow password file.
 	 */
-
 	for (spe = __spw_get_head (); spe; spe = spe->next) {
+		/*
+		 * Do not treat lines which were missing in gshadow
+		 * and were added earlier.
+		 */
+		if (spe->line == NULL)
+			continue;
+
 		/*
 		 * If this is a NIS line, skip it. You can't "know" what NIS
 		 * is going to do without directly asking NIS ...
 		 */
-
 		if (spe->line[0] == '+' || spe->line[0] == '-')
 			continue;
 
@@ -418,22 +450,18 @@ int main (int argc, char **argv)
 		 * have no (struct spwd) entry because they couldn't be
 		 * parsed properly.
 		 */
-
 		if (!spe->eptr) {
-
 			/*
 			 * Tell the user this entire line is bogus and ask
 			 * them to delete it.
 			 */
-
 			printf (_("invalid shadow password file entry\n"));
-			printf (_("delete line `%s'? "), spe->line);
+			printf (_("delete line '%s'? "), spe->line);
 			errors++;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-
 			if (!yes_or_no ())
 				continue;
 
@@ -443,11 +471,10 @@ int main (int argc, char **argv)
 			 * When done, it skips back to the top of the loop
 			 * to try out the next list element.
 			 */
-
 		      delete_spw:
 			SYSLOG ((LOG_INFO, "delete shadow line `%s'",
 				 spe->line));
-			deleted++;
+			changed++;
 
 			__spw_del_entry (spe);
 			continue;
@@ -456,27 +483,23 @@ int main (int argc, char **argv)
 		/*
 		 * Shadow password structure is good, start using it.
 		 */
-
 		spw = spe->eptr;
 
 		/*
 		 * Make sure this entry has a unique name.
 		 */
-
 		for (tspe = __spw_get_head (); tspe; tspe = tspe->next) {
 			const struct spwd *ent = tspe->eptr;
 
 			/*
 			 * Don't check this entry
 			 */
-
 			if (tspe == spe)
 				continue;
 
 			/*
 			 * Don't check invalid entries.
 			 */
-
 			if (!ent)
 				continue;
 
@@ -487,15 +510,13 @@ int main (int argc, char **argv)
 			 * Tell the user this entry is a duplicate of
 			 * another and ask them to delete it.
 			 */
-
-			puts (_("duplicate shadow password entry\n"));
-			printf (_("delete line `%s'? "), spe->line);
+			printf (_("duplicate shadow password entry\n"));
+			printf (_("delete line '%s'? "), spe->line);
 			errors++;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-
 			if (yes_or_no ())
 				goto delete_spw;
 		}
@@ -504,22 +525,19 @@ int main (int argc, char **argv)
 		 * Make sure this entry exists in the /etc/passwd
 		 * file.
 		 */
-
 		if (!pw_locate (spw->sp_namp)) {
-
 			/*
 			 * Tell the user this entry has no matching
 			 * /etc/passwd entry and ask them to delete it.
 			 */
-
-			puts (_("no matching password file entry\n"));
-			printf (_("delete line `%s'? "), spe->line);
+			printf (_("no matching password file entry in %s\n"),
+				pwd_file);
+			printf (_("delete line '%s'? "), spe->line);
 			errors++;
 
 			/*
 			 * prompt the user to delete the entry or not
 			 */
-
 			if (yes_or_no ())
 				goto delete_spw;
 		}
@@ -527,7 +545,6 @@ int main (int argc, char **argv)
 		/*
 		 * Warn if last password change in the future.  --marekm
 		 */
-
 		if (!quiet && spw->sp_lstchg > time ((time_t *) 0) / SCALE) {
 			printf (_
 				("user %s: last password change in the future\n"),
@@ -539,11 +556,10 @@ int main (int argc, char **argv)
       shadow_done:
 
 	/*
-	 * All done. If there were no deletions we can just abandon any
+	 * All done. If there were no change we can just abandon any
 	 * changes to the files.
 	 */
-
-	if (deleted) {
+	if (changed) {
 	      write_and_bye:
 		if (!pw_close ()) {
 			fprintf (stderr, _("%s: cannot update file %s\n"),
@@ -564,17 +580,17 @@ int main (int argc, char **argv)
 	/*
 	 * Don't be anti-social - unlock the files when you're done.
 	 */
-
 	if (is_shadow)
 		spw_unlock ();
 	(void) pw_unlock ();
 
+	nscd_flush_cache ("passwd");
+
 	/*
 	 * Tell the user what we did and exit.
 	 */
-
 	if (errors)
-		printf (deleted ?
+		printf (changed ?
 			_("%s: the files have been updated\n") :
 			_("%s: no changes\n"), Prog);
 
